# Generated by Django 5.0.3 on 2024-04-02 15:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_markov", "0002_alter_markovtextmodel_created_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="markovtextmodel",
            name="compiled",
            field=models.BooleanField(
                default=True,
                help_text="Is this model currently stored in compiled state?",
            ),
        ),
    ]
