# Generated by Django 5.0.3 on 2024-03-22 12:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_markov", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="markovtextmodel",
            name="created",
            field=models.DateTimeField(
                auto_now_add=True, help_text="When the model was created."
            ),
        ),
        migrations.AlterField(
            model_name="markovtextmodel",
            name="data",
            field=models.JSONField(
                blank=True, help_text="The compiled text model as JSON.", null=True
            ),
        ),
        migrations.AlterField(
            model_name="markovtextmodel",
            name="modified",
            field=models.DateTimeField(
                auto_now=True, help_text="Last modification of the record."
            ),
        ),
        migrations.DeleteModel(
            name="MarkovStats",
        ),
    ]
