#
# urls.py
#
# Copyright (c) 2024 Daniel Andrlik
# All rights reserved.
#
# SPDX-License-Identifier: BSD-3-Clause
#

"""Markov URL Configuration

There is nothing required here.
"""

app_name = "django_markov"
urlpatterns = []
