# Generated by Django 5.0.3 on 2024-04-02 16:31

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("django_markov", "0003_markovtextmodel_compiled"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="markovtextmodel",
            name="compiled",
        ),
    ]
